#include <Zeolite.h>
#include "zbin.h"

#include <zFunc.h>

//
// raw binary files
//

long zbin_Read(const char* lpFileName, ZLIST hContainerList) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.Read")) {
		zeoReportError("zbin_Read error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_Read error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hContainerList)) {
		zeoReportError("zbin_Read error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_Read error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_Read error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
	
}
long zbin_Write(const char* lpFileName, ZVAR hVar) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.Write")) {
		zeoReportError("zbin_Write error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_Write error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hVar)) {
		zeoReportError("zbin_Write error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_Write error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_Write error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}

//
// gzip files
//

long zbin_ReadGzip(const char* lpFileName, ZLIST hContainerList) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.ReadGzip")) {
		zeoReportError("zbin_ReadGzip error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_ReadGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hContainerList)) {
		zeoReportError("zbin_ReadGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_ReadGzip error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_ReadGzip error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
	
}
long zbin_WriteGzip(const char* lpFileName, ZVAR hVar) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.WriteGzip")) {
		zeoReportError("zbin_WriteGzip error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_WriteGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hVar)) {
		zeoReportError("zbin_WriteGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_WriteGzip error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_WriteGzip error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}

//
// raw buffers
//

long zbin_ReadBuffer(ZBUF hBuffer, ZLIST hContainerList) {
											
	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.ReadBuffer")) {
		zeoReportError("zbin_ReadBuffer error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(0, VarID_hvar, &hBuffer)) {
		zeoReportError("zbin_ReadBuffer error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hContainerList)) {
		zeoReportError("zbin_ReadBuffer error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_ReadBuffer error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_ReadBuffer error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}


long zbin_WriteBuffer(ZBUF hBuffer, ZVAR hVar) {
											
	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.WriteBuffer")) {
		zeoReportError("zbin_WriteBuffer error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(0, VarID_hvar, &hBuffer)) {
		zeoReportError("zbin_WriteBuffer error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hVar)) {
		zeoReportError("zbin_WriteBuffer error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_WriteBuffer error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_WriteBuffer error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}
