#include <Zeolite.h>
#include "zCalcHF.h"
#include <zVar.h>
#include <zStr.h>
#include <zMap.h>
#include <zFunc.h>

bool zCalcHF_Perlin(ZMAP pHF, float FeatureScale, int nOctaves, float Persistence, float MeanAlt, float AltRange, int NoiseKey, bool ShowProgress) {

	
	CzFunc f;
	if(!f.GetFunc("calc.HF.Perlin")) {
		zeoReportError("zCalcHF_Perlin error:\r\n - cannot get perlin function");
		return false;
	}
	
	long argid = 0;
	if(!f.m_args.SetItemValueI(argid++, VarID_hvar, &pHF)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_float, &FeatureScale)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_int, &nOctaves)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_float, &Persistence)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_float, &MeanAlt)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_float, &AltRange)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_int, &NoiseKey)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_bool, &ShowProgress)) {
		return false;
	}



	if(!f.Execute2()) {
		zeoReportError("zCalcHF_Perlin error:\r\n - call to calc.HF.Perlin failed");
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zCalcHF_Perlin error:\r\n - cannot get function return value");
		return false;
	}

	return rval;
}

bool zCalcHF_Fractal(ZMAP pHF, int PowerOfTwo, float Roughness, float AltRange, bool WrapFlag, bool ShowProgress) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.Fractal")) {
		zeoReportError("zCalcHF_Fractal error:\r\n - cannot get fractal function");
		return false;
	}
	
	long argid = 0;
	if(!f.m_args.SetItemValueI(argid++, VarID_hvar, &pHF)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_int, &PowerOfTwo)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_float, &Roughness)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_float, &AltRange)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_bool, &WrapFlag)) {
		return false;
	}
	if(!f.m_args.SetItemValueI(argid++, VarID_bool, &ShowProgress)) {
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zCalcHF_Fractal error:\r\n - call to calc.HF.Fractal failed");
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zCalcHF_Fractal error:\r\n - cannot get function return value");
		return false;
	}

	return rval;
}

bool zCalcHF_CurvFilter(ZMAP pHF, long nLevels, long nLevelsPerBlock, float Threshold, bool ShowProgress) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.CurvFilter")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevels))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevelsPerBlock))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &Threshold))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	ZMAP hMaskMap = NULL; // note, we won't set the mask map in this version
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &hMaskMap))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_Clamp(ZMAP pHF, float MinAlt, float MaxAlt, bool ShowProgress) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.ClampMap")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &MinAlt))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &MaxAlt))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_Shift(ZMAP pHF, float AltShift, bool ShowProgress) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.ShiftMap")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &AltShift))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_Scale(ZMAP pHF, float Factor, bool ShowProgress) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.ScaleMap")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &Factor))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_Power(ZMAP pHF, float Index, bool ShowProgress) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.PowerFilter")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &Index))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_Min(ZMAP pHF, float MinAlt, bool ShowProgress) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.MinFilter")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &MinAlt))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_Max(ZMAP pHF, float MaxAlt, bool ShowProgress) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.MaxFilter")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &MaxAlt))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}


bool zCalcHF_Inflate(ZMAP pHF, ZMAP pDM, long nLevels, float FractalFactor) {
	CzFunc f;
	if(!f.GetFunc("calc.HF.Inflate")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevels))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &FractalFactor))
		return false;

	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_InflateMosaic(ZMAP pMapSrc, ZMAP pMapDest, const char* lpFileName, const char* lpMapName, long TileSize, ZFORMAT pFormat, ZMAP pDM, long nLevels, float FractalFactor) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.InflateMosaic")) {
		zeoReportError("zCalcHF_InflateMosaic error:\r\n - cannot get 'calc.HF.InflateMosaic' function");
		return false;
	}

	CStringA TempStr;

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pMapSrc)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pMapDest)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!zstr_SetText(f.m_args.GetItem(ArgID++), lpFileName)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!zstr_SetText(f.m_args.GetItem(ArgID++), lpMapName)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &TileSize)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pFormat)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevels)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &FractalFactor)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		zeoReportError(TempStr);
		return false;
	}

	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zCalcHF_InflateMosaic error:\r\n - call to 'calc.HF.InflateMosaic' failed");
		return false;
	}

	bool b = false;
	if(!f.m_rval.GetValue(VarID_bool, &b)) {
		zeoReportError("zCalcHF_InflateMosaic error:\r\n - cannot get function return value");
		TempStr.Format("Type = %d\r\n", f.m_rval.GetTypeID());
		zeoReportError(TempStr);
		return false;
	}

	return b;
}

bool zCalcHF_PeakPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.PeakPass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
		
bool zCalcHF_MountainPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.MountainPass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_VolcanoPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.VolcanoPass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_PlateauPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.PlateauPass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_TerracePass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.TerracePass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_FileOverlayPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.FileOverlayPass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_ThermalPass(ZMAP pHF, ZMAP pDM, int nLevelMultiplier, int LevelOffset, int MaxLength, float S, float GradThreshold) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.ThermalPass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevelMultiplier))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &LevelOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &MaxLength))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &S))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &GradThreshold))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_ChannelPass(ZMAP pHF, ZMAP pDM, int nLevelMultiplier, int LevelOffset, int MinLength, int MaxLength, float DeltaHF, float MemDecay, bool BlurDir) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.ChannelPass")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pDM))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevelMultiplier))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &LevelOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &MinLength))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &MaxLength))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &DeltaHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &MemDecay))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &BlurDir))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_GenPeakOverlay(ZMAP pHF, int Radius, float Height, int ShapeMode) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.GenPeakOverlay")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &Radius))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &Height))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &ShapeMode))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_CombineMapsEx(ZMAP pHF, ZMAP pOverlay, int MixMode, int xOffset, int yOffset, int xOffsetMode, int yOffsetMode) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.CombineMapsEx")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pOverlay))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &MixMode))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &xOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &yOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &xOffsetMode))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &yOffsetMode))
		return false;
		
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}


bool zCalcHF_CombineMaps2(ZMAP pHF, ZMAP pOverlay, ZMAP pMaskMap, int MixMode, int xOffset, int yOffset, int xOffsetMode, int yOffsetMode, unsigned long Flags) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.CombineMaps2")) {
		return false;
	}

	long ArgID = 0;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pOverlay))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_hvar, &pMaskMap))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &MixMode))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &xOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &yOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &xOffsetMode))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &yOffsetMode))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_uint, &Flags))
		return false;
		
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}