#include "Zeolite.h"
#include "zbuffer.h"

CzBuffer::CzBuffer(void) {
	Create();

	// direct interface variables
	m_DirectInterfaceFlag = false;
	m_nItems = 0;
	m_ItemSize = 0;
	m_pMem = NULL;
}

CzBuffer::~CzBuffer(void) {
}

bool CzBuffer::Create() {
	return CzVar::Create(VarID_buffer);
}
bool CzBuffer::CreateShared(const char* lpVarName) {
	return CzVar::CreateShared(VarID_buffer, lpVarName);
}

bool CzBuffer::Init(long nItems, long ItemSize) {
	if(!m_hVar)	return false;

	return zbuffer_Init(m_hVar, nItems, ItemSize);
}
bool CzBuffer::InitByType(long nItems, long ItemTypeID) {
	if(!m_hVar)	return false;

	return zbuffer_InitByType(m_hVar, nItems, ItemTypeID);
}
bool CzBuffer::Free() {
	if(!m_hVar)	return false;
	
	return zbuffer_Free(m_hVar);
}
long CzBuffer::nItems() {
	if(!m_hVar)	return 0;
	
	return zbuffer_nItems(m_hVar);
}
long CzBuffer::MemSize() {
	if(!m_hVar)	return 0;
	
	return zbuffer_MemSize(m_hVar);
}
long CzBuffer::ItemSize() {
	if(!m_hVar)	return 0;
	
	return zbuffer_ItemSize(m_hVar);
}
long CzBuffer::ItemType() {
	if(!m_hVar)	return 0;
	
	return zbuffer_ItemType(m_hVar);
}
void* CzBuffer::GetPtr() {
	if(!m_hVar)	return NULL;
	
	return (void*)zbuffer_GetPtr(m_hVar);
}
void* CzBuffer::GetAt(long index) {
	if(!m_hVar)	return NULL;

	if(index<0 || index>=nItems())
		return NULL;
	
	return (void*)((char*)zbuffer_GetPtr(m_hVar)+index*ItemSize());
}
bool CzBuffer::GetValue(long index, void* pValue) {

	if(m_DirectInterfaceFlag) {
		memcpy(pValue, m_pMem + index * m_ItemSize, m_ItemSize);
		return true;
	}

	if(!m_hVar)	return false;
	
	return zbuffer_GetValue(m_hVar, index, pValue);
}
bool CzBuffer::SetValue(long index, void* pValue) {

	if(m_DirectInterfaceFlag) {
		memcpy(m_pMem + index * m_ItemSize, pValue, m_ItemSize);
		return true;
	}

	if(!m_hVar)	return false;
	
	return zbuffer_SetValue(m_hVar, index, pValue);
}

// direct interface
bool CzBuffer::OpenDirectInterface() {
	m_DirectInterfaceFlag = false;

	m_nItems = nItems();
	if(!m_nItems) {
		zeoReportError("CzBuffer::OpenDirectInterface error:\r\n - invalid item count");
		return false;
	}

	m_ItemSize = ItemSize();
	if(!m_ItemSize) {
		zeoReportError("CzBuffer::OpenDirectInterface error:\r\n - invalid item size");
		return false;
	}

	m_pMem = (char*)GetAt(0);
	if(!m_pMem) {
		zeoReportError("CzBuffer::OpenDirectInterface error:\r\n - cannot get mem handle");
		return false;
	}

	m_DirectInterfaceFlag = true;
	return true;
}
void CzBuffer::CloseDirectInterface() {
	m_DirectInterfaceFlag = false;
}