#include <Zeolite.h>
#include "zFile.h"
#include <zClasses.h>

bool zfile_GetDir(const char* lpPathName, ZSTR hResult) {

	if(!hResult) {
		zeoReportError("zfile_GetDir error:\r\n - invalid result handle");
		return false;
	}
	if(VarID_string!=zvar_GetTypeID(hResult)) {
		zeoReportError("zfile_GetDir error:\r\n - invalid result type");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("file.GetDir")) {
		zeoReportError("zfile_GetDir error:\r\n - cannot get 'file.GetDir' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpPathName)) {
		zeoReportError("zfile_GetDir error:\r\n - cannot set argument for 'file.GetDir' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_GetDir error:\r\n - call to function 'file.GetDir' failed");
		return false;
	}

	if(!zvar_CopyValue(hResult, f.m_rval)) {
		zeoReportError("zfile_GetDir error:\r\n - cannot copy rval from 'file.GetDir' function");
		return false;
	}

	return true;
}

bool zfile_GetExt(const char* lpFileName, ZSTR hResult) {

	if(!hResult) {
		zeoReportError("zfile_GetExt error:\r\n - invalid result handle");
		return false;
	}
	if(VarID_string!=zvar_GetTypeID(hResult)) {
		zeoReportError("zfile_GetExt error:\r\n - invalid result type");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("file.GetExt")) {
		zeoReportError("zfile_GetExt error:\r\n - cannot get 'file.GetExt' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpFileName)) {
		zeoReportError("zfile_GetExt error:\r\n - cannot set argument for 'file.GetExt' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_GetExt error:\r\n - call to function 'file.GetExt' failed");
		return false;
	}

	if(!zvar_CopyValue(hResult, f.m_rval)) {
		zeoReportError("zfile_GetExt error:\r\n - cannot copy rval from 'file.GetExt' function");
		return false;
	}

	return true;
}

bool zfile_TrimDir(const char* lpFileName, ZSTR hResult) {

	if(!hResult) {
		zeoReportError("zfile_TrimDir error:\r\n - invalid result handle");
		return false;
	}
	if(VarID_string!=zvar_GetTypeID(hResult)) {
		zeoReportError("zfile_TrimDir error:\r\n - invalid result type");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("file.TrimDir")) {
		zeoReportError("zfile_TrimDir error:\r\n - cannot get 'file.TrimDir' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpFileName)) {
		zeoReportError("zfile_TrimDir error:\r\n - cannot set argument for 'file.TrimDir' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_TrimDir error:\r\n - call to function 'file.TrimDir' failed");
		return false;
	}

	if(!zvar_CopyValue(hResult, f.m_rval)) {
		zeoReportError("zfile_TrimDir error:\r\n - cannot copy rval from 'file.TrimDir' function");
		return false;
	}

	return true;
}
bool zfile_TrimExt(const char* lpFileName, ZSTR hResult) {

	if(!hResult) {
		zeoReportError("zfile_TrimExt error:\r\n - invalid result handle");
		return false;
	}
	if(VarID_string!=zvar_GetTypeID(hResult)) {
		zeoReportError("zfile_TrimExt error:\r\n - invalid result type");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("file.TrimExt")) {
		zeoReportError("zfile_TrimExt error:\r\n - cannot get 'file.TrimExt' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpFileName)) {
		zeoReportError("zfile_TrimExt error:\r\n - cannot set argument for 'file.TrimExt' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_TrimExt error:\r\n - call to function 'file.TrimExt' failed");
		return false;
	}

	if(!zvar_CopyValue(hResult, f.m_rval)) {
		zeoReportError("zfile_TrimExt error:\r\n - cannot copy rval from 'file.TrimExt' function");
		return false;
	}

	return true;
}
bool zfile_TrimEllipses(const char* lpPathName, ZSTR hResult) {

	if(!hResult) {
		zeoReportError("zfile_TrimEllipses error:\r\n - invalid result handle");
		return false;
	}
	if(VarID_string!=zvar_GetTypeID(hResult)) {
		zeoReportError("zfile_TrimEllipses error:\r\n - invalid result type");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("file.TrimEllipses")) {
		zeoReportError("zfile_TrimEllipses error:\r\n - cannot get 'file.TrimEllipses' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpPathName)) {
		zeoReportError("zfile_TrimEllipses error:\r\n - cannot set argument for 'file.TrimEllipses' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_TrimEllipses error:\r\n - call to function 'file.TrimEllipses' failed");
		return false;
	}

	if(!zvar_CopyValue(hResult, f.m_rval)) {
		zeoReportError("zfile_TrimEllipses error:\r\n - cannot copy rval from 'file.TrimEllipses' function");
		return false;
	}

	return true;
}
bool zfile_TrimPath(const char* lpFileName, const char* lpRootPath, ZSTR hResult) {

	if(!hResult) {
		zeoReportError("zfile_TrimPath error:\r\n - invalid result handle");
		return false;
	}
	if(VarID_string!=zvar_GetTypeID(hResult)) {
		zeoReportError("zfile_TrimPath error:\r\n - invalid result type");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("file.TrimPath")) {
		zeoReportError("zfile_TrimPath error:\r\n - cannot get 'file.TrimPath' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpFileName)) {
		zeoReportError("zfile_TrimPath error:\r\n - cannot set argument for 'file.TrimPath' function");
		return false;
	}
	if(!zstr_SetText(f.GetArgI(1), lpRootPath)) {
		zeoReportError("zfile_TrimPath error:\r\n - cannot set argument for 'file.TrimPath' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_TrimPath error:\r\n - call to function 'file.TrimPath' failed");
		return false;
	}

	if(!zvar_CopyValue(hResult, f.m_rval)) {
		zeoReportError("zfile_TrimPath error:\r\n - cannot copy rval from 'file.TrimPath' function");
		return false;
	}

	return true;
}

bool zfile_FileExists(const char* lpFileName) {

	CzFunc f;
	if(!f.GetFunc("file.FileExists")) {
		zeoReportError("zfile_FileExists error:\r\n - cannot get 'file.FileExists' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpFileName)) {
		zeoReportError("zfile_FileExists error:\r\n - cannot set argument for 'file.FileExists' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_FileExists error:\r\n - call to function 'file.FileExists' failed");
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zfile_FileExists error:\r\n - cannot get rval from 'file.FileExists' function");
		return false;
	}

	return rval;
}


bool zfile_DirExists(const char* lpDirName) {

	CzFunc f;
	if(!f.GetFunc("file.DirExists")) {
		zeoReportError("zfile_DirExists error:\r\n - cannot get 'file.DirExists' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpDirName)) {
		zeoReportError("zfile_DirExists error:\r\n - cannot set argument for 'file.DirExists' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_DirExists error:\r\n - call to function 'file.DirExists' failed");
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zfile_DirExists error:\r\n - cannot get rval from 'file.DirExists' function");
		return false;
	}

	return rval;
}


bool zfile_CreateDir(const char* lpDirName) {
	CzFunc f;
	if(!f.GetFunc("file.CreateDir")) {
		zeoReportError("zfile_CreateDir error:\r\n - cannot get 'file.CreateDir' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpDirName)) {
		zeoReportError("zfile_CreateDir error:\r\n - cannot set argument for 'file.CreateDir' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_CreateDir error:\r\n - call to function 'file.CreateDir' failed");
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zfile_CreateDir error:\r\n - cannot get rval from 'file.CreateDir' function");
		return false;
	}

	return rval;
}

bool zfile_DeleteDir(const char* lpDirName) {
	CzFunc f;
	if(!f.GetFunc("file.DeleteDir")) {
		zeoReportError("zfile_DeleteDir error:\r\n - cannot get 'file.DeleteDir' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpDirName)) {
		zeoReportError("zfile_DeleteDir error:\r\n - cannot set argument for 'file.DeleteDir' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_DeleteDir error:\r\n - call to function 'file.DeleteDir' failed");
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zfile_DeleteDir error:\r\n - cannot get rval from 'file.DeleteDir' function");
		return false;
	}

	return rval;
}


bool zfile_IsAbsolutePath(const char* lpPathName) {

	CzFunc f;
	if(!f.GetFunc("file.IsAbsolutePath")) {
		zeoReportError("zfile_IsAbsolutePath error:\r\n - cannot get 'file.IsAbsolutePath' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpPathName)) {
		zeoReportError("zfile_IsAbsolutePath error:\r\n - cannot set argument for 'file.IsAbsolutePath' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_IsAbsolutePath error:\r\n - call to function 'file.IsAbsolutePath' failed");
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zfile_IsAbsolutePath error:\r\n - cannot get rval from 'file.IsAbsolutePath' function");
		return false;
	}

	return rval;
}

bool zfile_FindFile(const char* lpStartDir, const char* lpFileName, unsigned long Flags, ZLIST hResult) {

	if(!hResult) {
		zeoReportError("zfile_FindFile error:\r\n - invalid handle to hResult");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("file.FindFile")) {
		zeoReportError("zfile_FindFile error:\r\n - cannot get 'file.FindFile' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpStartDir)) {
		zeoReportError("zfile_FindFile error:\r\n - cannot set argument for 'file.FindFile' function");
		return false;
	}
	if(!zstr_SetText(f.GetArgI(1), lpFileName)) {
		zeoReportError("zfile_FindFile error:\r\n - cannot set argument for 'file.FindFile' function");
		return false;
	}
	if(!f.SetArgValueI(2, VarID_uint, &Flags)) {
		zeoReportError("zfile_FindFile error:\r\n - cannot set argument for 'file.FindFile' function");
		return false;
	}
	if(!f.SetArgValueI(3, VarID_hvar, &hResult)) {
		zeoReportError("zfile_FindFile error:\r\n - cannot set argument for 'file.FindFile' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_FindFile error:\r\n - call to function 'file.FindFile' failed");
		return false;
	}
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zfile_FindFile error:\r\n - cannot get rval");
		return false;
	}
	
	return rval;
}

bool zfile_CopyFile(const char* lpSrcPath, const char* lpDestPath, unsigned long Flags) {

	CzFunc f;
	if(!f.GetFunc("file.CopyFile")) {
		zeoReportError("zfile_CopyFile error:\r\n - cannot get 'file.CopyFile' function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpSrcPath)) {
		zeoReportError("zfile_CopyFile error:\r\n - cannot set argument for 'file.CopyFile' function");
		return false;
	}
	if(!zstr_SetText(f.GetArgI(1), lpDestPath)) {
		zeoReportError("zfile_CopyFile error:\r\n - cannot set argument for 'file.CopyFile' function");
		return false;
	}
	if(!zvar_SetValueEx(f.GetArgI(2), VarID_uint, &Flags)) {
		zeoReportError("zfile_CopyFile error:\r\n - cannot set argument for 'file.CopyFile' function");
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zfile_CopyFile error:\r\n - call to function 'file.CopyFile' failed");
		return false;
	}
	
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zfile_CopyFile error:\r\n - cannot get rval");
		return false;
	}

	return rval;
}