#include "Zeolite.h"
#include "zmap.h"

CzMap::CzMap(void) {
	Create();

	// direct interface variables
	m_DirectInterfaceFlag = false;
	m_nx = 0;
	m_ny = 0;
	m_nTilesX = 0;
	m_nTilesY = 0;
	m_nPixels = 0;
	m_TileSize = 0;
	m_PixelMemSize = 0;
	m_AllocMode = 0;
	m_pMem = NULL;
}

CzMap::~CzMap(void) {
}

bool CzMap::Create() {
	return CzVar::Create(VarID_map);
}
bool CzMap::CreateShared(const char* lpVarName) {
	return CzVar::CreateShared(VarID_map, lpVarName);
}

bool CzMap::Init(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag) {
	if(!m_hVar)	return false;

	return zmap_Init(m_hVar, nx, ny, MapTypeID, HorizScale, WrapFlag);
}
bool CzMap::Init2(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, unsigned long AllocMode, long TileSize) {
	if(!m_hVar)	return false;

	return zmap_Init2(m_hVar, nx, ny, MapTypeID, HorizScale, WrapFlag, AllocMode, TileSize);
}
bool CzMap::Free() {
	if(!m_hVar)	return false;

	m_DirectInterfaceFlag = false;

	return zmap_Free(m_hVar);
}
long CzMap::GetMapType() {
	if(!m_hVar)	return 0;

	return zmap_GetMapType(m_hVar);
}
long CzMap::GetPixelSize() {
	if(!m_hVar)	return 0;

	return zmap_GetPixelSize(m_hVar);
}
bool CzMap::GetWrapFlag() {
	if(!m_hVar)	return false;

	return zmap_GetWrapFlag(m_hVar);
}
bool CzMap::SetWrapFlag(bool WrapFlag) {
	if(!m_hVar)	return false;

	return zmap_SetWrapFlag(m_hVar, WrapFlag);
}
bool CzMap::GetBusyFlag() {
	if(!m_hVar)	return false;

	return zmap_GetBusyFlag(m_hVar);
}
bool CzMap::SetBusyFlag(bool BusyFlag) {
	if(!m_hVar)	return false;

	return zmap_SetBusyFlag(m_hVar, BusyFlag);
}
long CzMap::nx() {

	if(!m_hVar)	return 0;

	return zmap_GetWidth(m_hVar);
}
long CzMap::ny() {

	if(!m_hVar)	return 0;

	return zmap_GetHeight(m_hVar);
}
__int64 CzMap::nPixels() {

	if(!m_hVar)	return 0;

	return (__int64)nx() * (__int64)ny();
}
bool CzMap::GetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(MAPALLOC_RAM==m_AllocMode) {
			int k = x + y * m_nx;
			memcpy(pValue, (char*)m_pMem + k * m_PixelMemSize, m_PixelMemSize);
			return true;
		}
	}

	if(!m_hVar)	return false;

	return zmap_GetPixel(m_hVar, x, y, pValue);
}
bool CzMap::GetPixel(__int64 k, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(MAPALLOC_RAM==m_AllocMode) {
			memcpy(pValue, (char*)m_pMem + k * m_PixelMemSize, m_PixelMemSize);
			return true;
		}
	}

	if(!m_hVar)	return false;
	
	long tnx = nx();
	if(tnx<=0)	return false;
	long x = (long)(k%(__int64)tnx);
	long y = (long)(k/(__int64)tnx);

	return GetPixel(x, y, pValue);
}
bool CzMap::SetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(MAPALLOC_RAM==m_AllocMode) {
			int k = x + y * m_nx;
			memcpy((char*)m_pMem + k * m_PixelMemSize, pValue, m_PixelMemSize);
			return true;
		}
	}

	if(!m_hVar)	return false;

	return zmap_SetPixel(m_hVar, x, y, pValue);
}
bool CzMap::SetPixel(__int64 k, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(MAPALLOC_RAM==m_AllocMode) {
			memcpy((char*)m_pMem + k * m_PixelMemSize, pValue, m_PixelMemSize);
			return true;
		}
	}

	if(!m_hVar)	return false;
	
	long tnx = nx();
	if(tnx<=0)	return false;
	long x = (long)(k%(__int64)tnx);
	long y = (long)(k/(__int64)tnx);

	return SetPixel(x, y, pValue);
}
bool CzMap::GetMinMaxAlt(float& minval, float& maxval) {
	if(!m_hVar)	return false;

	return zmap_GetMinMaxAlt(m_hVar, minval, maxval);
}
bool CzMap::SetMinMaxAlt(float& minval, float& maxval) {
	if(!m_hVar)	return false;

	return zmap_SetMinMaxAlt(m_hVar, minval, maxval);
}
float CzMap::GetHorizScale() {
	if(!m_hVar)	return 1.0; // safe value, avoids div by zero

	return zmap_GetHorizScale(m_hVar);
}
bool CzMap::SetHorizScale(float scale) {
	if(!m_hVar)	return false;

	return zmap_SetHorizScale(m_hVar, scale);
}
bool CzMap::GetMosaicFlag() {
	if(!m_hVar)	return false;

	return zmap_GetMosaicFlag(m_hVar);
}
long CzMap::GetTileSize() {
	if(!m_hVar)	return 0;

	return zmap_GetTileSize(m_hVar);
}
long CzMap::GetTileCountX() {
	if(!m_hVar)	return 0;

	return zmap_GetTileCountX(m_hVar);
}
long CzMap::GetTileCountY() {
	if(!m_hVar)	return 0;

	return zmap_GetTileCountY(m_hVar);
}
bool CzMap::GetFlag(long FlagID) {
	if(!m_hVar)	return false;

	return zmap_GetFlag(m_hVar, FlagID);
}
bool CzMap::SetFlag(long FlagID, bool FlagValue) {
	if(!m_hVar)	return false;

	return zmap_SetFlag(m_hVar, FlagID, FlagValue);
}
unsigned long CzMap::GetAllocMode() {
	if(!m_hVar)	return 0;

	return zmap_GetAllocMode(m_hVar);
}
ZLIST CzMap::GetSettingsList() {
	if(!m_hVar)	return 0;

	return zmap_GetSettingsList(m_hVar);
}
bool CzMap::SaveFile(const char* lpFileName, ZFORMAT hFormat, bool SetFlags, bool ShowProgress) {
	if(!m_hVar)	return false;

	return zmap_SaveFile(m_hVar, lpFileName, hFormat, SetFlags, ShowProgress);
}
bool CzMap::LoadFile(const char* lpFileName, long MapTypeID, ZFORMAT hFormat, bool SetFlags, bool ShowProgress) {
	if(!m_hVar)	return false;

	return zmap_LoadFile(m_hVar, lpFileName, MapTypeID, hFormat, SetFlags, ShowProgress);
}
bool CzMap::GetNameInProject(ZVAR hStr) {
	if(!m_hVar)	return false;

	return zmap_GetNameInProject(m_hVar, hStr);
}

bool CzMap::InitMosaic(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName) {
	if(!m_hVar)	return false;

	return zmap_InitMosaic(m_hVar, nx, ny, MapTypeID, HorizScale, WrapFlag, lpMosaicFileName, TileSize, hFormat, lpProjMapName);
}
bool CzMap::LoadMosaic(const char* lpMosaicFileName) {
	if(!m_hVar)	return false;

	return zmap_LoadMosaic(m_hVar, lpMosaicFileName);
}
bool CzMap::SaveMosaic() {
	if(!m_hVar)	return false;

	return zmap_SaveMosaic(m_hVar);
}

ZFORMAT CzMap::GetFormat() {
	if(!m_hVar)	return 0;

	return zmap_GetFormat(m_hVar);
}
bool CzMap::SetFormat(ZFORMAT hFormat) {
	if(!m_hVar)	return false;

	return zmap_SetFormat(m_hVar, hFormat);
}
const char* CzMap::GetFilename() {
	if(!m_hVar)	return 0;

	return zmap_GetFilename(m_hVar);
}
bool CzMap::CombineMosaic() {
	if(!m_hVar)	return 0;

	return zmap_CombineMosaic(m_hVar);
}
bool CzMap::SplitToMosaic(const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName) {
	if(!m_hVar)	return 0;

	return zmap_SplitToMosaic(m_hVar, lpMosaicFileName, TileSize, hFormat, lpProjMapName);
}
bool CzMap::SaveMosaicAs(const char* lpFileName) {
	if(!m_hVar)	return 0;

	return zmap_SaveMosaicAs(m_hVar, lpFileName);
}
bool CzMap::ExportMap(const char* lpFileName, ZFORMAT hFormat, long nx, long ny) {
	if(!m_hVar)	return 0;

	return zmap_ExportMap(m_hVar, lpFileName, hFormat, nx, ny);
}
bool CzMap::ExportMosaic(const char* lpFileName, ZFORMAT hFormat, long nx, long ny, long TileSize, const char* lpProjMapName) {
	if(!m_hVar)	return 0;

	return zmap_ExportMosaic(m_hVar, lpFileName, hFormat, nx, ny, TileSize, lpProjMapName);
}
void* CzMap::GetDataPtr() {
	if(!m_hVar)	return 0;

	return zmap_GetDataPtr(m_hVar);
}
bool CzMap::LinInterp(double dx, double dy, void* pValue) {
	if(!m_hVar)	return 0;

	return zmap_LinInterp(m_hVar, dx, dy, pValue);
}

bool CzMap::GenMipmaps(long ResStep, long MaxLevel, long TileSize) {
	if(!m_hVar)	return false;

	return zmap_GenMipmaps(m_hVar, ResStep, MaxLevel, TileSize);
}
bool CzMap::ClearMipmaps() {
	if(!m_hVar)	return false;

	return zmap_ClearMipmaps(m_hVar);
}
ZMAP CzMap::GetMipmapLevel(long MipLevel) {
	if(!m_hVar)	return 0;

	return zmap_GetMipmapLevel(m_hVar, MipLevel);
}
long CzMap::GetMipmapResStep() {
	if(!m_hVar)	return 0;

	return zmap_GetMipmapResStep(m_hVar);
}
long CzMap::GetMipmapMaxLevel() {
	if(!m_hVar)	return 0;

	return zmap_GetMipmapMaxLevel(m_hVar);
}

// direct interface
bool CzMap::OpenDirectInterface() {
	
	if(!m_hVar) {
		zeoReportError("CzMap::OpenDirectInterface error:\r\n - invalid var handle");
		return false;
	}

	m_AllocMode = GetAllocMode();
	if(MAPALLOC_RAM==m_AllocMode) {
		m_pMem = GetDataPtr();

		if(!m_pMem) {
			zeoReportError("CzMap::OpenDirectInterface error:\r\n - cannot get mem handle");
			return false;
		}
	} else {
		// fail quietly; you can't do direct access on a mosaic; use zmap_GetMosaicTile & friends
		return false;
	}
	
	m_nx = nx();
	m_ny = ny();
	m_TileSize = GetTileSize();
	m_nTilesX = GetTileCountX();
	m_nTilesY = GetTileCountY();
	m_nPixels = nPixels();
	m_PixelMemSize = GetPixelSize();

	m_DirectInterfaceFlag = true;

	return true;
}
void CzMap::CloseDirectInterface() {
	m_DirectInterfaceFlag = false;
}
