#ifndef __ZEOLITE_VECTORS_INCLUDED__
#define __ZEOLITE_VECTORS_INCLUDED__

#include <Zeolite_defines.h>

#pragma once

// setting values

vector3f	VectorSet3f(float x, float y, float z);
vector3f	VectorSet3f(vector3d v); // double to float

vector3d	VectorSet3d(double x, double y, double z);
vector3d	VectorSet3d(vector3f v); // float to double

vector4f	VectorSet4f(float a, float b, float c, float d);

// addition, subtraction

vector3f	VectorAdd3f(vector3f a, vector3f b);
vector3f	VectorSub3f(vector3f a, vector3f b);

vector3d	VectorAdd3d(vector3d a, vector3d b);
vector3d	VectorSub3d(vector3d a, vector3d b);

// scaling / negation

vector3f	VectorScale3f(vector3f v, float f);
vector3f	VectorNegate3f(vector3f v);

vector3d	VectorScale3d(vector3d v, double d);
vector3d	VectorNegate3d(vector3d v);

// length, normalisation

double		VectorNorm3f(vector3f v);
double		VectorLength3f(vector3f v);
vector3f	VectorNormalise3f(vector3f v);

double		VectorNorm3d(vector3d v);
double		VectorLength3d(vector3d v);
vector3d	VectorNormalise3d(vector3d v);

// scalar and cross-product

double		ScalarProd3f(vector3f a, vector3f b);
vector3f	CrossProd3f(vector3f a, vector3f b);

double		ScalarProd3d(vector3d a, vector3d b);
vector3d	CrossProd3d(vector3d a, vector3d b);

// matrix transforms

vector3f	VectorTransform3f(vector3f v, matrix3f m);
vector3d	VectorTransform3d(vector3d v, matrix3d m);

// matrix math

matrix3d	MatrixIdentity3d();
matrix3f	MatrixIdentity3f();

matrix3d	MatrixSet3d(matrix3f m); // float to double

vector3d	MatrixRow3d(matrix3d m, int RowID);
vector3d	MatrixColumn3d(matrix3d m, int ColumnID);

matrix3d	MatrixProd3d(matrix3d a, matrix3d);

matrix3d  	MatrixRotateX3d(matrix3d m, float angle);
matrix3d  	MatrixRotateY3d(matrix3d m, float angle);
matrix3d  	MatrixRotateZ3d(matrix3d m, float angle);

matrix3d	MatrixFromAE3d(float azim, float elev);
matrix3d	MatrixFromAER3d(float azim, float elev, float roll);

#endif