#pragma once
#include "zvar.h"

#include "zMeshMtl.h"
#include "zMeshFaceGroup.h"

class CzMesh :
	public CzVar
{
public:
	CzMesh(void);
	~CzMesh(void);

	bool Create();
	bool CreateShared(const char* lpVarName);

	bool CreateData(ZLIST hMeshTarget = NULL);  // if hTarget is valid, the mesh will attach to it (must be empty) and initialise contents
	virtual bool Attach(ZLIST hMesh); // will load the mesh if present

	bool LoadHandles();

	const char* GetMeshName();
	bool SetMeshName(const char* lpMeshName);

	bool GetAxes(matrix3f** ppAxes);
	bool GetOffset(vector3f** ppOffset);
	bool GetScale(vector3f** ppScale);

	bool SetCoords(ZBUF hVertBuf, ZBUF hNormBuf, ZBUF hTexBuf);

	ZBUF GetVertBuf() { return m_hVar?m_hVertBuf:NULL; }
	ZBUF GetNormBuf() { return m_hVar?m_hNormBuf:NULL; }
	ZBUF GetTexBuf() { return m_hVar?m_hTexBuf:NULL; }
	ZLIST GetInfoEx() { return m_hVar?m_hInfoEx:NULL; }

	long nFaceGroups();
	long NewFaceGroup(CzMeshFaceGroup &mfg, const char* lpFaceGroupName = NULL); // retuns index, -1 is error
	bool GetFaceGroup(long index, CzMeshFaceGroup &mfg);
	long FindFaceGroup(const char* lpFaceGroupName, long StartAt = 0);
	bool DeleteFaceGroup(long index);
	
	long nMaterials();
	long NewMaterial(CzMeshMtl &mtl, const char* lpMtlName = NULL); // retuns index, -1 is error
	bool GetMaterial(long index, CzMeshMtl &mtl);
	long FindMaterial(const char* lpMtlName, long StartAt = 0);
	bool DeleteMaterial(long index);

	long nSubMeshes();
	long NewSubMesh(CzMesh &mesh, const char* lpSubMeshName = NULL); // retuns index, -1 is error
	bool GetSubMesh(long index, CzMesh &mesh);
	long FindSubMesh(const char* lpSubMeshName, long StartAt = 0);
	bool DeleteSubMesh(long index);

	// check buffer types, indices, etc.

	bool Validate();

protected:

	ZSTR m_hName;
	ZVAR m_hAxes;
	ZVAR m_hOffset;
	ZVAR m_hScale;
	ZBUF m_hVertBuf;
	ZBUF m_hNormBuf;
	ZBUF m_hTexBuf;
	ZLIST m_hFaceGroupList;
	ZLIST m_hMtlList;
	ZLIST m_hSubMeshList;
	ZLIST m_hRenderData; // temporary rendering data, which is not saved or loaded from files. Only the rendering app knows about the structure of the contents.
	ZLIST m_hInfoEx; // options not understood by base mesh class, but may be useful for renderer or file format

};