#include "Zeolite.h"
#include "zMeshMtl.h"

CzMeshMtl::CzMeshMtl(void) {
	
	m_hAmbCol = NULL;
	m_hDiffCol = NULL;
	m_hSpecCol = NULL;
	
	m_hSpecPower = NULL;

	m_hAmbImageFile = NULL;
	m_hDiffImageFile = NULL;
	m_hSpecImageFile = NULL;

	m_hInfoEx = NULL;
}

CzMeshMtl::~CzMeshMtl(void) {
}

bool CzMeshMtl::CreateData(ZLIST hTarget) {  // if hTarget is valid, the mesh will attach to it (must be empty) and initialise contents

	if(hTarget) {
		if(!CzVar::Attach(hTarget)) {
			return false;
		}
	}

	if(!m_hVar) {
		return false;
	}

	if(zlist_nItems(m_hVar)) {
		return false;
	}

	// root level items follow
	int MtlVer = 1;
	if(!zvar_SetValue(zlist_CreateItem(m_hVar, VarID_int, "MtlVer"), &MtlVer)) {
		return false;
	}

	// colours
	if(! (m_hAmbCol = zlist_CreateItem(m_hVar, VarID_vector4f, "AmbCol"))) {
		return false;
	}
	if(! (m_hDiffCol = zlist_CreateItem(m_hVar, VarID_vector4f, "DiffCol"))) {
		return false;
	}
	if(! (m_hSpecCol = zlist_CreateItem(m_hVar, VarID_vector4f, "SpecCol"))) {
		return false;
	}
	
	// images
	if(! (m_hAmbImageFile = zlist_CreateItem(m_hVar, VarID_string, "AmbImgageFile"))) {
		return false;
	}
	if(! (m_hDiffImageFile = zlist_CreateItem(m_hVar, VarID_string, "DiffImgageFile"))) {
		return false;
	}
	if(! (m_hSpecImageFile = zlist_CreateItem(m_hVar, VarID_string, "SpecImgageFile"))) {
		return false;
	}

	// specular power
	if(! (m_hSpecPower = zlist_CreateItem(m_hVar, VarID_float, "SpecPower"))) {
		return false;
	}

	// extra data
	if(! (m_hInfoEx  = zlist_CreateItem(m_hVar, VarID_varlist, "InfoEx"))) {
		return false;
	}

	return true;
}
bool CzMeshMtl::Attach(ZVAR hVar) {

	if(!CzVar::Attach(hVar)) {
		return false;
	}

	int argid = 0;

	// get mesh version
	int MtlVer = -1;
	if(!zvar_GetValueEx(zlist_GetItemI(m_hVar, argid++), VarID_int, &MtlVer)) {
		Release();
		return false;
	}

	switch(MtlVer) {
	case -1:
		Release();
		return false;
	default:
		Release();
		return false;
	case 1:
		{
			// colours
			if(!zvar_IsType((m_hAmbCol = zlist_GetItemI(m_hVar, argid++)), VarID_vector4f)) {
				return false;
			}
			if(!zvar_IsType( (m_hDiffCol = zlist_GetItemI(m_hVar, argid++)), VarID_vector4f)) {
				return false;
			}
			if(!zvar_IsType( (m_hSpecCol = zlist_GetItemI(m_hVar, argid++)), VarID_vector4f)) {
				return false;
			}
			
			// images
			if(!zvar_IsType( (m_hAmbImageFile = zlist_GetItemI(m_hVar, argid++)), VarID_string)) {
				return false;
			}
			if(!zvar_IsType( (m_hDiffImageFile = zlist_GetItemI(m_hVar, argid++)), VarID_string)) {
				return false;
			}
			if(!zvar_IsType( (m_hSpecImageFile = zlist_GetItemI(m_hVar, argid++)), VarID_string)) {
				return false;
			}

			// specular power
			if(!zvar_IsType( (m_hSpecPower = zlist_GetItemI(m_hVar, argid++)), VarID_float)) {
				return false;
			}

			// extra data
			if(!zvar_IsType((m_hInfoEx  = zlist_GetItemI(m_hVar, argid++)), VarID_varlist)) {
				return false;
			}
		}
		break;
	}

	return true;
}

bool CzMeshMtl::GetAmbCol(vector4f** ppColour) {
	if(!ppColour)	return false;
	vector4f* pValue = (vector4f*)zvar_GetDataPtr(m_hAmbCol);
	if(!pValue) return false;

	*ppColour = pValue;
	return true;
}
bool CzMeshMtl::GetDiffCol(vector4f** ppColour) {
	if(!ppColour)	return false;
	vector4f* pValue = (vector4f*)zvar_GetDataPtr(m_hDiffCol);
	if(!pValue) return false;

	*ppColour = pValue;
	return true;
}
bool CzMeshMtl::GetSpecCol(vector4f** ppColour) {
	if(!ppColour)	return false;
	vector4f* pValue = (vector4f*)zvar_GetDataPtr(m_hSpecCol);
	if(!pValue) return false;

	*ppColour = pValue;
	return true;
}
bool CzMeshMtl::GetSpecPower(float** ppValue) {
	if(!ppValue)	return false;
	float* pValue = (float*)zvar_GetDataPtr(m_hSpecPower);
	if(!pValue) return false;

	*ppValue = pValue;
	return true;
}

bool CzMeshMtl::SetAmbient(vector4f col, const char* lpImageFile) {
	if(!m_hVar) {
		return false;
	}

	if(!zvar_SetValueEx(m_hAmbCol, VarID_vector4f, &col)) {
		return false;
	}

	if(!zstr_SetText(m_hAmbImageFile, lpImageFile)) {
		return false;
	}

	return true;
}
bool CzMeshMtl::SetDiffuse(vector4f col, const char* lpImageFile) {
	if(!m_hVar) {
		return false;
	}

	if(!zvar_SetValueEx(m_hDiffCol, VarID_vector4f, &col)) {
		return false;
	}

	if(!zstr_SetText(m_hDiffImageFile, lpImageFile)) {
		return false;
	}

	return true;
}
bool CzMeshMtl::SetSpecular(vector4f col, const char* lpImageFile, float SpecPower) {
	if(!m_hVar) {
		return false;
	}

	if(!zvar_SetValueEx(m_hSpecCol, VarID_vector4f, &col)) {
		return false;
	}

	if(!zstr_SetText(m_hSpecImageFile, lpImageFile)) {
		return false;
	}

	if(!zvar_SetValueEx(m_hSpecPower, VarID_float, &SpecPower)) {
		return false;
	}

	return true;
}